//*****************************************************************************
//
// io.c - I/O routines for the enet_io example application.
//
// Copyright (c) 2007-2011 Texas Instruments Incorporated.  All rights reserved.
// Software License Agreement
// 
// Texas Instruments (TI) is supplying this software for use solely and
// exclusively on TI's microcontroller products. The software is owned by
// TI and/or its suppliers, and is protected under applicable copyright
// laws. You may not combine this software with "viral" open-source
// software in order to form a larger program.
// 
// THIS SOFTWARE IS PROVIDED "AS IS" AND WITH ALL FAULTS.
// NO WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT
// NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. TI SHALL NOT, UNDER ANY
// CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
// DAMAGES, FOR ANY REASON WHATSOEVER.
// 
// This is part of revision 8049 of the DK-LM3S9B96 Firmware Package.
//
//*****************************************************************************

#include "inc/hw_ints.h"
#include "inc/hw_memmap.h"
#include "inc/hw_pwm.h"
#include "inc/hw_types.h"
#include "driverlib/gpio.h"
#include "driverlib/pwm.h"
#include "driverlib/sysctl.h"
#include "driverlib/timer.h"
#include "driverlib/interrupt.h"
#include "driverlib/rom.h"
#include "utils/ustdlib.h"
#include "lwip/def.h"
#include "string.h"
#include "io.h"

//*****************************************************************************
//
// Hardware connection for the user LED.
//
//*****************************************************************************
#define LED_PORT_BASE GPIO_PORTG_BASE
#define LED_PIN GPIO_PIN_2


//*****************************************************************************
//
// Stores the city name or zip/postal code for the city which the weather 
// forcast will be requested.
//
//*****************************************************************************
char g_pucWeatherCity[40] = {"fort%20worth"};
char g_pucWeatherUnits[] = {"Fahrenheit"};
unsigned int g_bWeatherApplicationUpdated = false;

//*****************************************************************************
//
// Initialize the IO used in this demo
//
//*****************************************************************************
void
io_init(void)
{
    //
    // Configure Port F0 for as an output for the status LED.
    //
    ROM_GPIOPinTypeGPIOOutput(LED_PORT_BASE, LED_PIN);

    //
    // Initialize LED to OFF (0)
    //
    ROM_GPIOPinWrite(LED_PORT_BASE, LED_PIN, 0);

    /*//
    // Enable the peripherals used by this example.
    //
    ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_TIMER2);

    //
    // Configure the timer used to pace the animation.
    //
    ROM_TimerConfigure(TIMER2_BASE, TIMER_CFG_PERIODIC);
    
    //
    // Setup for 1 second periodic interrupt
    //
    ROM_TimerLoadSet(TIMER2_BASE, TIMER_BOTH, ROM_SysCtlClockGet());

    //
    // Setup the interrupts for the timer timeouts.
    //
    ROM_IntEnable(INT_TIMER2A);
    ROM_TimerIntEnable(TIMER2_BASE, TIMER_TIMA_TIMEOUT);*/
}


//*****************************************************************************
//
// Set the status LED on or off.
//
//*****************************************************************************
void
io_set_led(tBoolean bOn)
{
    //
    // Turn the LED on or off as requested.
    //
    GPIOPinWrite(LED_PORT_BASE, LED_PIN, bOn ? LED_PIN : 0);
	//MAP_GPIOPinWrite(GPIO_PORTG_BASE, GPIO_PIN_2, 0);    
}

//*****************************************************************************
//
// Return LED state
//
//*****************************************************************************
void
io_get_ledstate(char * pcBuf, int iBufLen)
{
    //
    // Get the state of the LED
    //
    if(GPIOPinRead(LED_PORT_BASE, LED_PIN))
    {
        usnprintf(pcBuf, iBufLen, "ON");
    }
    else
    {
        usnprintf(pcBuf, iBufLen, "OFF");
    }

}

//*****************************************************************************
//
// Return LED state as an integer, 1 on, 0 off.
//
//*****************************************************************************
int
io_is_led_on(void)
{
    //
    // Get the state of the LED
    //
    if(GPIOPinRead(LED_PORT_BASE, LED_PIN))
    {
        return(1);
    }
    else
    {
        return(0);
    }
}

//*****************************************************************************
//
// Set the current city string as an ASCII string.
//
//*****************************************************************************
void io_set_city_string(char* ucBuf)
{
  strcpy(g_pucWeatherCity, ucBuf);
  g_bWeatherApplicationUpdated = true;
}

//*****************************************************************************
//
// Get the current city string as an ASCII string.
//
//*****************************************************************************
void
io_get_city_string(char *pcBuf)
{
  strcpy(pcBuf, g_pucWeatherCity);
}

//*****************************************************************************
//
// Set the current Temperature Units string as an ASCII string.
//
//*****************************************************************************
void io_set_units_string(char* ucBuf)
{
  strcpy(g_pucWeatherUnits, ucBuf);
  g_bWeatherApplicationUpdated = true;
}

//*****************************************************************************
//
// Get the current Temperature Units string as an ASCII string.
//
//*****************************************************************************
void
io_get_units_string(char *pcBuf)
{
  strcpy(pcBuf, g_pucWeatherUnits);
}

//*****************************************************************************
//
// Get the flag that is set when the webserver has updated the Weather 
// application.
//
//*****************************************************************************
unsigned int
io_get_weather_application_update(void)
{
  return(g_bWeatherApplicationUpdated);
}

//*****************************************************************************
//
// Set the flag that is set when the webserver has updated the Weather 
// application. Used to clear flag
//
//*****************************************************************************
void
io_set_weather_application_update(unsigned int bValue)
{
  g_bWeatherApplicationUpdated = bValue;
}
